@@ -30,8 +30,9 @@ import shutil
 import site
 import subprocess
 import sys
+from collections.abc import Iterator
 from contextlib import contextmanager
-from typing import TYPE_CHECKING, Any, Callable, Iterator, TextIO
+from typing import TYPE_CHECKING, Any, Callable, TextIO
 
 from ansible_compat.config import ansible_version
 from ansible_compat.prerun import get_cache_dir
@@ -50,6 +51,7 @@ from ansiblelint.color import (
 from ansiblelint.config import get_version_warning, options
 from ansiblelint.constants import EXIT_CONTROL_C_RC, GIT_CMD, LOCK_TIMEOUT_RC
 from ansiblelint.file_utils import abspath, cwd, normpath
+from ansiblelint.loaders import load_ignore_txt
 from ansiblelint.skip_utils import normalize_tag
 from ansiblelint.version import __version__
 
@@ -131,23 +133,18 @@ def initialize_options(arguments: list[str] | None = None) -> None:
 def _do_list(rules: RulesCollection) -> int:
     # On purpose lazy-imports to avoid pre-loading Ansible
     # pylint: disable=import-outside-toplevel
-    from ansiblelint.generate_docs import (
-        rules_as_docs,
-        rules_as_md,
-        rules_as_rich,
-        rules_as_str,
-    )
+    from ansiblelint.generate_docs import rules_as_md, rules_as_rich, rules_as_str
 
     if options.list_rules:
-
         _rule_format_map: dict[str, Callable[..., Any]] = {
-            "plain": rules_as_str,
-            "rich": rules_as_rich,
+            "brief": rules_as_str,
+            "full": rules_as_rich,
             "md": rules_as_md,
-            "docs": rules_as_docs,
         }
 
-        console.print(_rule_format_map[options.format](rules), highlight=False)
+        console.print(
+            _rule_format_map.get(options.format, rules_as_str)(rules), highlight=False
+        )
         return 0
 
     if options.list_tags:
@@ -215,6 +212,11 @@ def main(argv: list[str] | None = None) -> int:  # noqa: C901
     _logger.debug("Options: %s", options)
     _logger.debug(os.getcwd())
 
+    if options.progressive:
+        _logger.warning(
+            "Progressive mode is deprecated and will be removed in next major version, use ignore files instead: https://ansible-lint.readthedocs.io/configuring/#ignoring-rules-for-entire-files"
+        )
+
     if not options.offline:
         # pylint: disable=import-outside-toplevel
         from ansiblelint.schemas import refresh_schemas
@@ -281,6 +283,14 @@ def main(argv: list[str] | None = None) -> int:  # noqa: C901
     if options.strict and result.matches:
         mark_as_success = False
 
+    # Remove skip_list items from the result
+    result.matches = [m for m in result.matches if m.tag not in app.options.skip_list]
+    # Mark matches as ignored inside ignore file
+    ignore_map = load_ignore_txt(options.ignore_file)
+    for match in result.matches:
+        if match.tag in ignore_map[match.filename]:
+            match.ignored = True
+
     app.render_matches(result.matches)
 
     _perform_mockings_cleanup()
@@ -369,6 +379,7 @@ def path_inject() -> None:
             paths[idx] = os.path.expanduser(path)
             expanded = True
     if expanded:  # pragma: no cover
+        # flake8: noqa: T201
         print(
             "WARNING: PATH altered to expand ~ in it. Read https://stackoverflow.com/a/44704799/99834 and correct your system configuration.",
             file=sys.stderr,
@@ -387,6 +398,7 @@ def path_inject() -> None:
         inject_paths.append(py_path)
 
     if inject_paths:
+        # flake8: noqa: T201
         print(
             f"WARNING: PATH altered to include {', '.join(inject_paths)} :: This is usually a sign of broken local setup, which can cause unexpected behaviors.",
             file=sys.stderr,
