@@ -17,7 +17,7 @@ _logger = logging.getLogger(__name__)
 LOAD_FAILURE_MD = """\
 # load-failure
 
-Linter failed to process a YAML file, probably because it is either:
+"Linter failed to process a file, possible invalid file. Possible reasons:
 
 * contains unsupported encoding (only UTF-8 is supported)
 * not an Ansible file
@@ -175,11 +175,11 @@ class LoadingFailureRule(BaseRule):
     """Failed to load or parse file."""
 
     id = "load-failure"
-    description = "Linter failed to process a YAML file, possible not an Ansible file."
+    description = "Linter failed to process a file, possible invalid file."
     severity = "VERY_HIGH"
     tags = ["core", "unskippable"]
     version_added = "v4.3.0"
-    help = LOAD_FAILURE_MD
+    _help = LOAD_FAILURE_MD
     _order = 0
 
 
