@@ -23,21 +23,12 @@ _logger = logging.getLogger(__name__)
 
 
 CACHE_DIR = (
-    os.path.expanduser(os.environ.get("XDG_CONFIG_CACHE", "~/.cache")) + "/ansible-lint"
+    os.path.expanduser(os.environ.get("XDG_CACHE_HOME", "~/.cache")) + "/ansible-lint"
 )
 
 DEFAULT_WARN_LIST = [
-    "avoid-implicit",
     "experimental",
-    "fqcn[action]",
-    "fqcn[redirect]",
     "jinja[spacing]",  # warning until we resolve all reported false-positives
-    "name[casing]",
-    "name[play]",
-    "name[prefix]",
-    "role-name",
-    "warning[empty-playbook]",  # because ansible considers it warning only
-    "role-name[path]",  # too new
 ]
 
 DEFAULT_KINDS = [
@@ -56,6 +47,7 @@ DEFAULT_KINDS = [
     {"galaxy": "**/galaxy.yml"},  # Galaxy collection meta
     {"reno": "**/releasenotes/*/*.{yaml,yml}"},  # reno release notes
     {"tasks": "**/tasks/**/*.{yaml,yml}"},
+    {"rulebook": "**/rulebooks/*.{yml,yaml"},
     {"playbook": "**/playbooks/*.{yml,yaml}"},
     {"playbook": "**/*playbook*.{yml,yaml}"},
     {"role": "**/roles/*/"},
@@ -73,9 +65,10 @@ DEFAULT_KINDS = [
     {"requirements": "**/requirements.{yaml,yml}"},  # v2 and v1
     {"playbook": "**/molecule/*/*.{yaml,yml}"},  # molecule playbooks
     {"yaml": "**/{.ansible-lint,.yamllint}"},
-    {"changelog": "**/changelog.yml"},
+    {"changelog": "**/changelogs/changelog.yaml"},
     {"yaml": "**/*.{yaml,yml}"},
     {"yaml": "**/.*.{yaml,yml}"},
+    {"sanity-ignore-file": "**/tests/sanity/ignore-*.txt"},
 ]
 
 BASE_KINDS = [
@@ -99,6 +92,8 @@ BASE_KINDS = [
 
 PROFILES = yaml_from_file(Path(__file__).parent / "data" / "profiles.yml")
 
+LOOP_VAR_PREFIX = "^(__|{role}_)"
+
 options = Namespace(
     cache_dir=None,
     colored=True,
@@ -106,7 +101,7 @@ options = Namespace(
     cwd=".",
     display_relative_path=True,
     exclude_paths=[],
-    format="rich",
+    format="brief",
     lintables=[],
     list_rules=False,
     list_tags=False,
@@ -135,6 +130,7 @@ options = Namespace(
     rules={},  # Placeholder to set and keep configurations for each rule.
     profile=None,
     task_name_prefix="{stem} | ",
+    sarif_file=None,
 )
 
 # Used to store detected tag deprecations
