@@ -9,7 +9,7 @@ from typing import TYPE_CHECKING, Any
 
 import black
 import jinja2
-from ansible.errors import AnsibleError, AnsibleFilterError, AnsibleParserError
+from ansible.errors import AnsibleError, AnsibleParserError
 from ansible.parsing.yaml.objects import AnsibleUnicode
 from jinja2.exceptions import TemplateSyntaxError
 
@@ -17,8 +17,9 @@ from ansiblelint.constants import LINE_NUMBER_KEY
 from ansiblelint.file_utils import Lintable
 from ansiblelint.rules import AnsibleLintRule
 from ansiblelint.skip_utils import get_rule_skips_from_line
+from ansiblelint.text import has_jinja
 from ansiblelint.utils import parse_yaml_from_file, template
-from ansiblelint.yaml_utils import nested_items_path
+from ansiblelint.yaml_utils import deannotate, nested_items_path
 
 if TYPE_CHECKING:
     from ansiblelint.errors import MatchError
@@ -29,6 +30,24 @@ KEYWORDS_WITH_IMPLICIT_TEMPLATE = ("changed_when", "failed_when", "until", "when
 
 Token = namedtuple("Token", "lineno token_type value")
 
+ignored_re = re.compile(
+    "|".join(
+        [
+            r"^Object of type method is not JSON serializable",
+            r"^Unexpected templating type error occurred on",
+            r"^obj must be a list of dicts or a nested dict$",
+            r"^the template file (.*) could not be found for the lookup$",
+            r"could not locate file in lookup",
+            r"unable to locate collection",
+            r"^Error in (.*)is undefined$",
+            r"^Mandatory variable (.*) not defined.$",
+            r"is undefined",
+            r"Unrecognized type <<class 'ansible.template.AnsibleUndefined'>> for (.*) filter <value>$",
+        ]
+    ),
+    flags=re.MULTILINE | re.DOTALL,
+)
+
 
 class JinjaRule(AnsibleLintRule):
     """Rule that looks inside jinja2 templates."""
@@ -51,85 +70,96 @@ class JinjaRule(AnsibleLintRule):
         """Generate error message."""
         return self._tag2msg[tag].format(value=value, reformatted=reformatted)
 
-    # pylint: disable=too-many-branches
+    # pylint: disable=too-many-branches,too-many-locals
     def matchtask(  # noqa: C901
         self, task: dict[str, Any], file: Lintable | None = None
-    ) -> bool | str | MatchError:
-        for key, v, _ in nested_items_path(task):
-            if isinstance(v, str):
-
-                try:
-                    template(
-                        basedir=file.dir if file else ".",
-                        value=v,
-                        variables={},
-                        fail_on_error=True,  # we later decide which ones to ignore or not
-                    )
-                # ValueError RepresenterError
-                except AnsibleError as exc:
-                    bypass = False
-                    orig_exc = exc.orig_exc if getattr(exc, "orig_exc", None) else exc
-                    match = self._ansible_error_re.match(
-                        getattr(orig_exc, "message", str(orig_exc))
-                    )
-                    if (
-                        isinstance(exc, AnsibleFilterError)
-                        or "unable to locate collection" in orig_exc.message
-                    ):
-                        bypass = True
-                    elif re.match(
-                        r"^the template file (.*) could not be found for the lookup$",
-                        orig_exc.message,
-                    ) or re.match(r"could not locate file in lookup", orig_exc.message):
-                        bypass = True
-                    elif isinstance(orig_exc, AnsibleParserError):
-                        # "An unhandled exception occurred while running the lookup plugin '...'. Error was a <class 'ansible.errors.AnsibleParserError'>, original message: Invalid filename: 'None'. Invalid filename: 'None'"
-
-                        # An unhandled exception occurred while running the lookup plugin 'template'. Error was a <class 'ansible.errors.AnsibleError'>, original message: the template file ... could not be found for the lookup. the template file ... could not be found for the lookup
-
-                        # ansible@devel (2.14) new behavior:
-                        # AnsibleError(TemplateSyntaxError): template error while templating string: Could not load "ipwrap": 'Invalid plugin FQCN (ansible.netcommon.ipwrap): unable to locate collection ansible.netcommon'. String: Foo {{ buildset_registry.host | ipwrap }}. Could not load "ipwrap": 'Invalid plugin FQCN (ansible.netcommon.ipwrap): unable to locate collection ansible.netcommon'
-                        bypass = True
-                    elif (
-                        isinstance(orig_exc, (AnsibleError, TemplateSyntaxError))
-                        and match
-                    ):
-                        error = match.group("error")
-                        detail = match.group("detail")
-                        # string = match.group("string")
-                        if error.startswith("template error while templating string"):
-                            bypass = False
-                        elif detail.startswith("unable to locate collection"):
-                            _logger.debug("Ignored AnsibleError: %s", exc)
-                            bypass = True
-                        else:
-                            # breakpoint()
-                            bypass = False
-                    elif re.match(r"^lookup plugin (.*) not found$", exc.message):
-                        # lookup plugin 'template' not found
-                        bypass = True
-
-                    # AnsibleFilterError: 'obj must be a list of dicts or a nested dict'
-                    # AnsibleError: template error while templating string: expected token ':', got '}'. String: {{ {{ '1' }} }}
-                    # AnsibleError: template error while templating string: unable to locate collection ansible.netcommon. String: Foo {{ buildset_registry.host | ipwrap }}
-                    if not bypass:
-                        return self.create_matcherror(
-                            message=str(exc),
-                            linenumber=task[LINE_NUMBER_KEY],
-                            filename=file,
-                            tag=f"{self.id}[invalid]",
+    ) -> list[MatchError]:
+        result = []
+        try:
+            for key, v, path in nested_items_path(
+                task,
+                ignored_keys=("block", "ansible.builtin.block", "ansible.legacy.block"),
+            ):
+                if isinstance(v, str):
+                    try:
+                        template(
+                            basedir=file.dir if file else ".",
+                            value=v,
+                            variables=deannotate(task.get("vars", {})),
+                            fail_on_error=True,  # we later decide which ones to ignore or not
                         )
+                    # ValueError RepresenterError
+                    except AnsibleError as exc:
+                        bypass = False
+                        orig_exc = (
+                            exc.orig_exc if getattr(exc, "orig_exc", None) else exc
+                        )
+                        orig_exc_message = getattr(orig_exc, "message", str(orig_exc))
+                        match = self._ansible_error_re.match(
+                            getattr(orig_exc, "message", str(orig_exc))
+                        )
+                        if ignored_re.search(orig_exc_message):
+                            bypass = True
+                        elif isinstance(orig_exc, AnsibleParserError):
+                            # "An unhandled exception occurred while running the lookup plugin '...'. Error was a <class 'ansible.errors.AnsibleParserError'>, original message: Invalid filename: 'None'. Invalid filename: 'None'"
 
-                reformatted, details, tag = self.check_whitespace(v, key=key)
-                if reformatted != v:
-                    return self.create_matcherror(
-                        message=self._msg(tag=tag, value=v, reformatted=reformatted),
-                        linenumber=task[LINE_NUMBER_KEY],
-                        details=details,
-                        filename=file,
-                        tag=f"{self.id}[{tag}]",
+                            # An unhandled exception occurred while running the lookup plugin 'template'. Error was a <class 'ansible.errors.AnsibleError'>, original message: the template file ... could not be found for the lookup. the template file ... could not be found for the lookup
+
+                            # ansible@devel (2.14) new behavior:
+                            # AnsibleError(TemplateSyntaxError): template error while templating string: Could not load "ipwrap": 'Invalid plugin FQCN (ansible.netcommon.ipwrap): unable to locate collection ansible.netcommon'. String: Foo {{ buildset_registry.host | ipwrap }}. Could not load "ipwrap": 'Invalid plugin FQCN (ansible.netcommon.ipwrap): unable to locate collection ansible.netcommon'
+                            bypass = True
+                        elif (
+                            isinstance(orig_exc, (AnsibleError, TemplateSyntaxError))
+                            and match
+                        ):
+                            error = match.group("error")
+                            detail = match.group("detail")
+                            # string = match.group("string")
+                            if error.startswith(
+                                "template error while templating string"
+                            ):
+                                bypass = False
+                            elif detail.startswith("unable to locate collection"):
+                                _logger.debug("Ignored AnsibleError: %s", exc)
+                                bypass = True
+                            else:
+                                bypass = False
+                        elif re.match(r"^lookup plugin (.*) not found$", exc.message):
+                            # lookup plugin 'template' not found
+                            bypass = True
+
+                        # AnsibleFilterError: 'obj must be a list of dicts or a nested dict'
+                        # AnsibleError: template error while templating string: expected token ':', got '}'. String: {{ {{ '1' }} }}
+                        # AnsibleError: template error while templating string: unable to locate collection ansible.netcommon. String: Foo {{ buildset_registry.host | ipwrap }}
+                        if not bypass:
+                            result.append(
+                                self.create_matcherror(
+                                    message=str(exc),
+                                    linenumber=_get_error_line(task, path),
+                                    filename=file,
+                                    tag=f"{self.id}[invalid]",
+                                )
+                            )
+                            continue
+                    reformatted, details, tag = self.check_whitespace(
+                        v, key=key, lintable=file
                     )
-        return False
+                    if reformatted != v:
+                        result.append(
+                            self.create_matcherror(
+                                message=self._msg(
+                                    tag=tag, value=v, reformatted=reformatted
+                                ),
+                                linenumber=_get_error_line(task, path),
+                                details=details,
+                                filename=file,
+                                tag=f"{self.id}[{tag}]",
+                            )
+                        )
+        except Exception as exc:
+            _logger.info("Exception in JinjaRule.matchtask: %s", exc)
+            raise
+        return result
 
     def matchyaml(self, file: Lintable) -> list[MatchError]:
         """Return matches for variables defined in vars files."""
@@ -142,7 +172,9 @@ class JinjaRule(AnsibleLintRule):
             # pylint: disable=unused-variable
             for key, v, path in nested_items_path(data):
                 if isinstance(v, AnsibleUnicode):
-                    reformatted, details, tag = self.check_whitespace(v, key=key)
+                    reformatted, details, tag = self.check_whitespace(
+                        v, key=key, lintable=file
+                    )
                     if reformatted != v:
                         results.append(
                             self.create_matcherror(
@@ -173,13 +205,15 @@ class JinjaRule(AnsibleLintRule):
 
         self.env.lstrip_blocks = False
         self.env.trim_blocks = False
+        self.env.autoescape = True
+        self.env.newline_sequence = "\n"
         tokens = [
             Token(lineno=t[0], token_type=t[1], value=t[2]) for t in self.env.lex(text)
         ]
         new_text = self.unlex(tokens)
         if text != new_text:
             _logger.debug(
-                "Unable to perform full roundrip lex-unlex on jinja template (expected when '-' modifier is used): {text} -> {new_text}"
+                "Unable to perform full roundtrip lex-unlex on jinja template (expected when '-' modifier is used): {text} -> {new_text}"
             )
         return tokens
 
@@ -198,11 +232,12 @@ class JinjaRule(AnsibleLintRule):
 
     # pylint: disable=too-many-branches,too-many-statements,too-many-locals
     def check_whitespace(  # noqa: max-complexity: 13
-        self, text: str, key: str
+        self, text: str, key: str, lintable: Lintable | None = None
     ) -> tuple[str, str, str]:
         """Check spacing inside given jinja2 template string.
 
         We aim to match Python Black formatting rules.
+        :raises NotImplementedError: On few cases where valid jinja is not valid Python.
 
         :returns: (string, string, string)  reformatted text, detailed error, error tag
         """
@@ -229,17 +264,28 @@ class JinjaRule(AnsibleLintRule):
         implicit = False
 
         # implicit templates do not have the {{ }} wrapping
-        if key in KEYWORDS_WITH_IMPLICIT_TEMPLATE:
+        if (
+            key in KEYWORDS_WITH_IMPLICIT_TEMPLATE
+            and lintable
+            and lintable.kind
+            in (
+                "playbook",
+                "task",
+            )
+        ):
             implicit = True
             text = cook(text, implicit=implicit)
 
+        # don't try to lex strings that have no jinja inside them
+        if not has_jinja(text):
+            return text, "", "spacing"
+
         expr_str = None
         expr_type = None
         verb_skipped = True
         lineno = 1
         try:
             for token in self.lex(text):
-
                 if (
                     expr_type
                     and expr_type.startswith("{%")
@@ -260,7 +306,8 @@ class JinjaRule(AnsibleLintRule):
                     # pylint: disable=unsupported-membership-test
                     if isinstance(expr_str, str) and "\n" in expr_str:
                         raise NotImplementedError()
-                    expr_str = blacken(expr_str)
+                    leading_spaces = " " * (len(expr_str) - len(expr_str.lstrip()))
+                    expr_str = leading_spaces + blacken(expr_str.lstrip())
                     if tokens[
                         -1
                     ].token_type != "whitespace" and not expr_str.startswith(" "):
@@ -310,7 +357,6 @@ def blacken(text: str) -> str:
 
 
 if "pytest" in sys.modules:  # noqa: C901
-
     import pytest
 
     from ansiblelint.rules import RulesCollection  # pylint: disable=ungrouped-imports
@@ -319,7 +365,7 @@ if "pytest" in sys.modules:  # noqa: C901
     @pytest.fixture(name="error_expected_lines")
     def fixture_error_expected_lines() -> list[int]:
         """Return list of expected error lines."""
-        return [31, 34, 37, 40, 43, 46, 72]
+        return [33, 36, 39, 42, 45, 48, 74]
 
     # 21 68
     @pytest.fixture(name="lint_error_lines")
@@ -365,7 +411,8 @@ if "pytest" in sys.modules:  # noqa: C901
             pytest.param("", "", "spacing", id="1"),
             pytest.param("foo", "foo", "spacing", id="2"),
             pytest.param("{##}", "{# #}", "spacing", id="3"),
-            pytest.param("{#  #}", "{# #}", "spacing", id="4"),
+            # we want to keep leading spaces as they might be needed for complex multiline jinja files
+            pytest.param("{#  #}", "{#  #}", "spacing", id="4"),
             pytest.param(
                 "{{-aaa|xx   }}foo\nbar{#some#}\n{%%}",
                 "{{- aaa | xx }}foo\nbar{# some #}\n{% %}",
@@ -540,13 +587,37 @@ if "pytest" in sys.modules:  # noqa: C901
                 "spacing",
                 id="42",
             ),
+            pytest.param(
+                "{{ lookup('file'   ,  '/tmp/non-existent',  errors='ignore') }}",
+                "{{ lookup('file', '/tmp/non-existent', errors='ignore') }}",
+                "spacing",
+                id="43",
+            ),
+            # https://github.com/ansible/ansible-lint/pull/3057
+            # since jinja 3.0.0, \r is converted to \n if the string has jinja in it
+            pytest.param(
+                "{{ 'foo' }}\r{{ 'bar' }}",
+                "{{ 'foo' }}\n{{ 'bar' }}",
+                "spacing",
+                id="44",
+            ),
+            # if we do not have any jinja constructs, we should keep original \r
+            # to match ansible behavior
+            pytest.param(
+                "foo\rbar",
+                "foo\rbar",
+                "spacing",
+                id="45",
+            ),
         ),
     )
     def test_jinja(text: str, expected: str, tag: str) -> None:
         """Tests our ability to spot spacing errors inside jinja2 templates."""
         rule = JinjaRule()
 
-        reformatted, details, returned_tag = rule.check_whitespace(text, key="name")
+        reformatted, details, returned_tag = rule.check_whitespace(
+            text, key="name", lintable=Lintable("playbook.yml")
+        )
         assert tag == returned_tag, details
         assert expected == reformatted
 
@@ -573,24 +644,60 @@ if "pytest" in sys.modules:  # noqa: C901
     def test_jinja_implicit(text: str, expected: str, tag: str) -> None:
         """Tests our ability to spot spacing errors implicit jinja2 templates."""
         rule = JinjaRule()
-        reformatted, details, returned_tag = rule.check_whitespace(text, key="when")
+        # implicit jinja2 are working only inside playbooks and tasks
+        lintable = Lintable(name="playbook.yml", kind="playbook")
+        reformatted, details, returned_tag = rule.check_whitespace(
+            text, key="when", lintable=lintable
+        )
         assert tag == returned_tag, details
         assert expected == reformatted
 
+    @pytest.mark.parametrize(
+        ("lintable", "matches"),
+        (pytest.param("examples/playbooks/vars/rule_jinja_vars.yml", 0, id="0"),),
+    )
+    def test_jinja_file(lintable: str, matches: int) -> None:
+        """Tests our ability to process var filesspot spacing errors."""
+        collection = RulesCollection()
+        collection.register(JinjaRule())
+        errs = Runner(lintable, rules=collection).run()
+        assert len(errs) == matches
+        for err in errs:
+            assert isinstance(err, JinjaRule)
+            assert errs[0].tag == "jinja[invalid]"
+            assert errs[0].rule.id == "jinja"
+
     def test_jinja_invalid() -> None:
         """Tests our ability to spot spacing errors inside jinja2 templates."""
         collection = RulesCollection()
         collection.register(JinjaRule())
-        success = "examples/playbooks/rule-jinja-invalid.yml"
+        success = "examples/playbooks/rule-jinja-fail.yml"
         errs = Runner(success, rules=collection).run()
-        assert len(errs) == 1
-        assert errs[0].tag == "jinja[invalid]"
+        assert len(errs) == 2
+        assert errs[0].tag == "jinja[spacing]"
         assert errs[0].rule.id == "jinja"
+        assert errs[0].linenumber == 9
+        assert errs[1].tag == "jinja[invalid]"
+        assert errs[1].rule.id == "jinja"
+        assert errs[1].linenumber == 9
 
     def test_jinja_valid() -> None:
         """Tests our ability to parse jinja, even when variables may not be defined."""
         collection = RulesCollection()
         collection.register(JinjaRule())
-        success = "examples/playbooks/rule-jinja-valid.yml"
+        success = "examples/playbooks/rule-jinja-pass.yml"
         errs = Runner(success, rules=collection).run()
         assert len(errs) == 0
+
+
+def _get_error_line(task: dict[str, Any], path: list[str | int]) -> int:
+    """Return error line number."""
+    line = task[LINE_NUMBER_KEY]
+    ctx = task
+    for _ in path:
+        ctx = ctx[_]
+        if LINE_NUMBER_KEY in ctx:
+            line = ctx[LINE_NUMBER_KEY]
+    if not isinstance(line, int):
+        raise RuntimeError("Line number is not an integer")
+    return line
